@@ -27,9 +27,16 @@
 #include <polkit/polkit.h>
 #endif
 
-#include "cd-cleanup.h"
 #include "cd-common.h"
 
+#ifdef USE_POLKIT
+#ifndef PolkitAuthorizationResult_autoptr
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(PolkitAuthorizationResult, g_object_unref)
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(PolkitSubject, g_object_unref)
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(PolkitAuthority, g_object_unref)
+#endif
+#endif
+
 /**
  * cd_client_error_quark:
  **/
@@ -74,7 +81,7 @@ cd_main_get_sender_uid (GDBusConnection *connection,
 			GError **error)
 {
 	guint uid = G_MAXUINT;
-	_cleanup_variant_unref_ GVariant *value = NULL;
+	g_autoptr(GVariant) value = NULL;
 
 	/* call into DBus to get the user ID that issued the request */
 	value = g_dbus_connection_call_sync (connection,
@@ -103,7 +110,7 @@ cd_main_get_sender_pid (GDBusConnection *connection,
 			GError **error)
 {
 	guint pid = G_MAXUINT;
-	_cleanup_variant_unref_ GVariant *value = NULL;
+	g_autoptr(GVariant) value = NULL;
 
 	/* call into DBus to get the user ID that issued the request */
 	value = g_dbus_connection_call_sync (connection,
@@ -133,11 +140,11 @@ cd_main_sender_authenticated (GDBusConnection *connection,
 			      GError **error)
 {
 	guint uid;
-	_cleanup_error_free_ GError *error_local = NULL;
+	g_autoptr(GError) error_local = NULL;
 #ifdef USE_POLKIT
-	_cleanup_object_unref_ PolkitAuthority *authority = NULL;
-	_cleanup_object_unref_ PolkitAuthorizationResult *result = NULL;
-	_cleanup_object_unref_ PolkitSubject *subject = NULL;
+	g_autoptr(PolkitAuthority) authority = NULL;
+	g_autoptr(PolkitAuthorizationResult) result = NULL;
+	g_autoptr(PolkitSubject) subject = NULL;
 #endif
 
 	/* uid 0 is allowed to do all actions */
@@ -223,7 +230,7 @@ cd_main_mkdir_with_parents (const gchar *filename, GError **error)
 {
 	/* ensure desination exists */
 	if (!g_file_test (filename, G_FILE_TEST_EXISTS)) {
-		_cleanup_object_unref_ GFile *file = NULL;
+		g_autoptr(GFile) file = NULL;
 		file = g_file_new_for_path (filename);
 		return g_file_make_directory_with_parents (file, NULL, error);
 	}
